@@ -32,15 +32,16 @@ import json
 import uuid
 import networkx as nx
 from abc import ABC, abstractmethod
-from typing import List, Dict, Any, Set
+from typing import List, Dict, Any, Set, Tuple
 
 import logging
 
 from fim.slivers.attached_components import ComponentSliver, AttachedComponentsInfo
-from fim.slivers.capacities_labels import Capacities, Labels
+from fim.slivers.capacities_labels import Capacities, Labels, ReservationInfo
 from fim.slivers.delegations import Delegation
 from fim.slivers.interface_info import InterfaceSliver, InterfaceInfo
 from fim.slivers.network_node import NodeSliver
+from fim.slivers.network_link import NetworkLinkSliver
 from fim.slivers.switch_fabric import SwitchFabricSliver, SwitchFabricInfo
 from fim.graph.abc_property_graph_constants import ABCPropertyGraphConstants
 
@@ -54,6 +55,29 @@ class ABCPropertyGraph(ABCPropertyGraphConstants):
     Graphs are distinguished by GraphID property on each node.
     """
 
+    SLIVER_PROPERTY_TO_GRAPH = {
+        "name": ABCPropertyGraphConstants.PROP_NAME,
+        "type": ABCPropertyGraphConstants.PROP_TYPE,
+        "capacities": ABCPropertyGraphConstants.PROP_CAPACITIES,
+        "labels": ABCPropertyGraphConstants.PROP_LABELS,
+        "capacity_delegations": ABCPropertyGraphConstants.PROP_CAPACITY_DELEGATIONS,
+        "label_delegations": ABCPropertyGraphConstants.PROP_LABEL_DELEGATIONS,
+        "capacity_allocations": ABCPropertyGraphConstants.PROP_CAPACITY_ALLOCATIONS,
+        "label_allocations": ABCPropertyGraphConstants.PROP_LABEL_ALLOCATIONS,
+        "reservation_info": ABCPropertyGraphConstants.PROP_RESERVATION_INFO,
+        "site": ABCPropertyGraphConstants.PROP_SITE,
+        # note lack of image type in this mapping
+        "image_ref": ABCPropertyGraphConstants.PROP_IMAGE_REF,
+        "management_ip": ABCPropertyGraphConstants.PROP_MGMT_IP,
+        "allocation_constraints": ABCPropertyGraphConstants.PROP_ALLOCATION_CONSTRAINTS,
+        "service_endpoint": ABCPropertyGraphConstants.PROP_SERVICE_ENDPOINT,
+        "details": ABCPropertyGraphConstants.PROP_DETAILS,
+        "layer": ABCPropertyGraphConstants.PROP_LAYER,
+        "technology": ABCPropertyGraphConstants.PROP_TECHNOLOGY,
+        "model": ABCPropertyGraphConstants.PROP_MODEL,
+        "node_map": ABCPropertyGraphConstants.PROP_NODE_MAP
+    }
+
     @abstractmethod
     def __init__(self, *, graph_id: str, importer, logger=None):
         """
@@ -125,6 +149,16 @@ class ABCPropertyGraph(ABCPropertyGraphConstants):
         :return:
         """
 
+    @abstractmethod
+    def unset_node_property(self, *, node_id: str, prop_name: str) -> None:
+        """
+        Unset/remove this property from a node. Not all properties can be
+        unset (e.g. graph id and node id cannot)
+        :param node_id:
+        :param prop_name:
+        :return:
+        """
+
     @abstractmethod
     def update_nodes_property(self, *, prop_name: str, prop_val: Any) -> None:
         """
@@ -156,6 +190,18 @@ class ABCPropertyGraph(ABCPropertyGraphConstants):
         :return:
         """
 
+    @abstractmethod
+    def unset_link_property(self, *, node_a: str, node_b: str, kind: str,
+                             prop_name: str) -> None:
+        """
+        Unset a property on a specified link
+        :param node_a:
+        :param node_b:
+        :param kind:
+        :param prop_name:
+        :return:
+        """
+
     @abstractmethod
     def update_link_properties(self, *, node_a: str, node_b: str, kind: str,
                                props: Dict[str, Any]) -> None:
@@ -175,6 +221,14 @@ class ABCPropertyGraph(ABCPropertyGraphConstants):
         :return:
         """
 
+    @abstractmethod
+    def get_all_nodes_by_class(self, *, label: str) -> List[str]:
+        """
+        Get a list of nodes of this class/label
+        :param label:
+        :return:
+        """
+
     @abstractmethod
     def serialize_graph(self) -> str:
         """
@@ -299,7 +353,7 @@ class ABCPropertyGraph(ABCPropertyGraphConstants):
     def __repr__(self):
         return f"Graph with id {self.graph_id}"
 
-    def _validate_json_property(self, node_id: str, prop_name: str, strict: bool = False) -> str:
+    def _validate_json_property(self, node_id: str, props: Dict, strict: bool = False) -> Tuple[str, str] or None:
         """
         Validate that JSON in a particular node in a particular property is valid or throw
         a JSONDecodeError exception. Strict set to true causes exception if property is not found.
@@ -309,25 +363,24 @@ class ABCPropertyGraph(ABCPropertyGraphConstants):
         :param strict
         :return:
         """
-        assert node_id is not None
-        assert prop_name is not None
-        _, props = self.get_node_properties(node_id=node_id)
-        if prop_name not in props.keys():
-            if strict:
-                # if property is not there, raise exception
-                raise PropertyGraphImportException(graph_id=self.graph_id, node_id=node_id,
-                                                   msg=f"Unable to find property {prop_name} on a node")
+        for prop_name in ABCPropertyGraphConstants.JSON_PROPERTY_NAMES:
+            if prop_name not in props.keys():
+                if strict:
+                    # if property is not there, raise exception
+                    raise PropertyGraphImportException(graph_id=self.graph_id, node_id=node_id,
+                                                       msg=f"Unable to find property {prop_name} on a node")
+                else:
+                    # if property is not there, just return
+                    continue
+            # try loading it as JSON. Exception may be thrown
+            if props[prop_name] is not None and props[prop_name] != "None":
+                try:
+                    json.loads(props[prop_name])
+                except json.decoder.JSONDecodeError:
+                    return prop_name, props[prop_name]
             else:
-                # if property is not there, just return
-                return None
-        # try loading it as JSON. Exception may be thrown
-        if props[prop_name] is not None and props[prop_name] != "None":
-            try:
-                json.loads(props[prop_name])
-            except json.decoder.JSONDecodeError:
-                return props[prop_name]
-        else:
-            return None
+                continue
+        return None
 
     def _validate_all_json_properties(self) -> None:
         """
@@ -339,11 +392,208 @@ class ABCPropertyGraph(ABCPropertyGraphConstants):
             raise PropertyGraphQueryException(graph_id=self.graph_id,
                                               node_id=None, msg="Unable to list nodes of graph")
         for node in nodes:
-            for prop in self.JSON_PROPERTY_NAMES:
-                prop_val = self._validate_json_property(node_id=node, prop_name=prop)
-                if prop_val is not None:
-                    raise PropertyGraphImportException(graph_id=self.graph_id, node_id=node,
-                                                       msg=f"Unable to parse JSON property {prop} with value {prop_val}")
+            _, props = self.get_node_properties(node_id=node)
+            prop_tuple = self._validate_json_property(node_id=node, props=props)
+            if prop_tuple is not None:
+                raise PropertyGraphImportException(graph_id=self.graph_id, node_id=node,
+                                                   msg=f"Unable to parse JSON property {prop_tuple[0]} "
+                                                       f"with value {prop_tuple[1]}")
+
+    @staticmethod
+    def map_sliver_property_to_graph(prop_name: str) -> str or None:
+        """
+        Map a property name in a sliver to a graph property or return None
+        :param prop_name:
+        :return:
+        """
+        return ABCPropertyGraph.SLIVER_PROPERTY_TO_GRAPH.get(prop_name, None)
+
+    @staticmethod
+    def node_sliver_to_graph_properties_dict(sliver: NodeSliver) -> Dict[str, str]:
+        """
+        This method knows how to map sliver fields to graph properties
+        :param sliver:
+        :return:
+        """
+        prop_dict = dict()
+
+        if sliver.resource_name is not None:
+            prop_dict[ABCPropertyGraph.PROP_NAME] = sliver.resource_name
+        if sliver.resource_type is not None:
+            prop_dict[ABCPropertyGraph.PROP_TYPE] = str(sliver.resource_type)
+        if sliver.resource_model is not None:
+            prop_dict[ABCPropertyGraph.PROP_MODEL] = sliver.resource_model
+        if sliver.capacities is not None:
+            prop_dict[ABCPropertyGraph.PROP_CAPACITIES] = sliver.capacities.to_json()
+        if sliver.labels is not None:
+            prop_dict[ABCPropertyGraph.PROP_LABELS] = sliver.labels.to_json()
+        if sliver.capacity_delegations is not None:
+            prop_dict[ABCPropertyGraph.PROP_CAPACITY_DELEGATIONS] = \
+                Delegation.from_list_to_json(sliver.capacity_delegations)
+        if sliver.label_delegations is not None:
+            prop_dict[ABCPropertyGraph.PROP_LABEL_DELEGATIONS] = \
+                Delegation.from_list_to_json(sliver.label_delegations)
+        if sliver.capacity_allocations is not None:
+            prop_dict[ABCPropertyGraph.PROP_CAPACITY_ALLOCATIONS] = sliver.capacity_allocations.to_json()
+        if sliver.label_allocations is not None:
+            prop_dict[ABCPropertyGraph.PROP_LABEL_ALLOCATIONS] = sliver.label_allocations.to_json()
+        if sliver.reservation_info is not None:
+            prop_dict[ABCPropertyGraph.PROP_RESERVATION_INFO] = sliver.reservation_info.to_json()
+        if sliver.site is not None:
+            prop_dict[ABCPropertyGraph.PROP_SITE] = sliver.site
+        if sliver.image_ref is not None and sliver.image_type is not None:
+            prop_dict[ABCPropertyGraph.PROP_IMAGE_REF] = sliver.image_ref + ',' + str(sliver.image_type)
+        if sliver.management_ip is not None:
+            prop_dict[ABCPropertyGraph.PROP_MGMT_IP] = str(sliver.management_ip)
+        if sliver.allocation_constraints is not None:
+            prop_dict[ABCPropertyGraph.PROP_ALLOCATION_CONSTRAINTS] = sliver.allocation_constraints
+        if sliver.service_endpoint is not None:
+            prop_dict[ABCPropertyGraph.PROP_SERVICE_ENDPOINT] = str(sliver.service_endpoint)
+        if sliver.details is not None:
+            prop_dict[ABCPropertyGraph.PROP_DETAILS] = sliver.details
+        if sliver.node_map is not None:
+            prop_dict[ABCPropertyGraph.PROP_NODE_MAP] = json.dumps(sliver.node_map)
+        return prop_dict
+
+    @staticmethod
+    def link_sliver_to_graph_properties_dict(sliver: NetworkLinkSliver) -> Dict[str, str]:
+        """
+        This method knows how to map sliver fields to graph properties
+        :param sliver:
+        :return:
+        """
+        prop_dict = dict()
+
+        if sliver.resource_name is not None:
+            prop_dict[ABCPropertyGraph.PROP_NAME] = sliver.resource_name
+        if sliver.resource_type is not None:
+            prop_dict[ABCPropertyGraph.PROP_TYPE] = str(sliver.resource_type)
+        if sliver.resource_model is not None:
+            prop_dict[ABCPropertyGraph.PROP_MODEL] = sliver.resource_model
+        if sliver.capacities is not None:
+            prop_dict[ABCPropertyGraph.PROP_CAPACITIES] = sliver.capacities.to_json()
+        if sliver.labels is not None:
+            prop_dict[ABCPropertyGraph.PROP_LABELS] = sliver.labels.to_json()
+        if sliver.capacity_delegations is not None:
+            prop_dict[ABCPropertyGraph.PROP_CAPACITY_DELEGATIONS] = \
+                Delegation.from_list_to_json(sliver.capacity_delegations)
+        if sliver.label_delegations is not None:
+            prop_dict[ABCPropertyGraph.PROP_LABEL_DELEGATIONS] = \
+                Delegation.from_list_to_json(sliver.label_delegations)
+        if sliver.capacity_allocations is not None:
+            prop_dict[ABCPropertyGraph.PROP_CAPACITY_ALLOCATIONS] = sliver.capacity_allocations.to_json()
+        if sliver.label_allocations is not None:
+            prop_dict[ABCPropertyGraph.PROP_LABEL_ALLOCATIONS] = sliver.label_allocations.to_json()
+        if sliver.reservation_info is not None:
+            prop_dict[ABCPropertyGraph.PROP_RESERVATION_INFO] = sliver.reservation_info.to_json()
+        if sliver.layer is not None:
+            prop_dict[ABCPropertyGraph.PROP_LAYER] = str(sliver.layer)
+        if sliver.technology is not None:
+            prop_dict[ABCPropertyGraph.PROP_TECHNOLOGY] = str(sliver.technology)
+        if sliver.details is not None:
+            prop_dict[ABCPropertyGraph.PROP_DETAILS] = sliver.details
+        if sliver.node_map is not None:
+            prop_dict[ABCPropertyGraph.PROP_NODE_MAP] = json.dumps(sliver.node_map)
+
+        return prop_dict
+
+    @staticmethod
+    def component_sliver_to_graph_properties_dict(sliver: ComponentSliver) -> Dict[str, str]:
+        """
+        This method knows how to map component sliver fields to graph properties
+        :param sliver:
+        :return:
+        """
+        prop_dict = dict()
+
+        if sliver.resource_name is not None:
+            prop_dict[ABCPropertyGraph.PROP_NAME] = sliver.resource_name
+        if sliver.resource_type is not None:
+            prop_dict[ABCPropertyGraph.PROP_TYPE] = str(sliver.resource_type)
+        if sliver.resource_model is not None:
+            prop_dict[ABCPropertyGraph.PROP_MODEL] = sliver.resource_model
+        if sliver.capacities is not None:
+            prop_dict[ABCPropertyGraph.PROP_CAPACITIES] = sliver.capacities.to_json()
+        if sliver.labels is not None:
+            prop_dict[ABCPropertyGraph.PROP_LABELS] = sliver.labels.to_json()
+        if sliver.capacity_delegations is not None:
+            prop_dict[ABCPropertyGraph.PROP_CAPACITY_DELEGATIONS] = \
+                Delegation.from_list_to_json(sliver.capacity_delegations)
+        if sliver.label_delegations is not None:
+            prop_dict[ABCPropertyGraph.PROP_LABEL_DELEGATIONS] = \
+                Delegation.from_list_to_json(sliver.label_delegations)
+        if sliver.details is not None:
+            prop_dict[ABCPropertyGraph.PROP_DETAILS] = sliver.details
+        if sliver.node_map is not None:
+            prop_dict[ABCPropertyGraph.PROP_NODE_MAP] = json.dumps(sliver.node_map)
+
+        return prop_dict
+
+    @staticmethod
+    def switch_fabric_sliver_to_graph_properties_dict(sliver: SwitchFabricSliver) -> Dict[str, str]:
+        """
+        This method knows how to map switch fabric sliver fields to graph properties
+        :param sliver:
+        :return:
+        """
+        prop_dict = dict()
+
+        if sliver.resource_name is not None:
+            prop_dict[ABCPropertyGraph.PROP_NAME] = sliver.resource_name
+        if sliver.resource_type is not None:
+            prop_dict[ABCPropertyGraph.PROP_TYPE] = str(sliver.resource_type)
+        if sliver.resource_model is not None:
+            prop_dict[ABCPropertyGraph.PROP_MODEL] = sliver.resource_model
+        if sliver.capacities is not None:
+            prop_dict[ABCPropertyGraph.PROP_CAPACITIES] = sliver.capacities.to_json()
+        if sliver.labels is not None:
+            prop_dict[ABCPropertyGraph.PROP_LABELS] = sliver.labels.to_json()
+        if sliver.capacity_delegations is not None:
+            prop_dict[ABCPropertyGraph.PROP_CAPACITY_DELEGATIONS] = \
+                Delegation.from_list_to_json(sliver.capacity_delegations)
+        if sliver.label_delegations is not None:
+            prop_dict[ABCPropertyGraph.PROP_LABEL_DELEGATIONS] = \
+                Delegation.from_list_to_json(sliver.label_delegations)
+        if sliver.layer is not None:
+            prop_dict[ABCPropertyGraph.PROP_LAYER] = str(sliver.layer)
+        if sliver.details is not None:
+            prop_dict[ABCPropertyGraph.PROP_DETAILS] = sliver.details
+        if sliver.node_map is not None:
+            prop_dict[ABCPropertyGraph.PROP_NODE_MAP] = json.dumps(sliver.node_map)
+
+        return prop_dict
+
+    @staticmethod
+    def interface_sliver_to_graph_properties_dict(sliver: InterfaceSliver) -> Dict[str, str]:
+        """
+        This method knows how to map interface sliver fields to graph properties
+        :param sliver:
+        :return:
+        """
+        prop_dict = dict()
+
+        if sliver.resource_name is not None:
+            prop_dict[ABCPropertyGraph.PROP_NAME] = sliver.resource_name
+        if sliver.resource_type is not None:
+            prop_dict[ABCPropertyGraph.PROP_TYPE] = str(sliver.resource_type)
+        if sliver.resource_model is not None:
+            prop_dict[ABCPropertyGraph.PROP_MODEL] = sliver.resource_model
+        if sliver.capacities is not None:
+            prop_dict[ABCPropertyGraph.PROP_CAPACITIES] = sliver.capacities.to_json()
+        if sliver.labels is not None:
+            prop_dict[ABCPropertyGraph.PROP_LABELS] = sliver.labels.to_json()
+        if sliver.capacity_delegations is not None:
+            prop_dict[ABCPropertyGraph.PROP_CAPACITY_DELEGATIONS] = \
+                Delegation.from_list_to_json(sliver.capacity_delegations)
+        if sliver.label_delegations is not None:
+            prop_dict[ABCPropertyGraph.PROP_LABEL_DELEGATIONS] = \
+                Delegation.from_list_to_json(sliver.label_delegations)
+        if sliver.details is not None:
+            prop_dict[ABCPropertyGraph.PROP_DETAILS] = sliver.details
+        if sliver.node_map is not None:
+            prop_dict[ABCPropertyGraph.PROP_NODE_MAP] = json.dumps(sliver.node_map)
+
+        return prop_dict
 
     @staticmethod
     def node_sliver_from_graph_properties_dict(d: Dict[str, str]) -> NodeSliver:
@@ -353,10 +603,17 @@ class ABCPropertyGraph(ABCPropertyGraphConstants):
             image_type = None
         else:
             image_ref, image_type = d[ABCPropertyGraph.PROP_IMAGE_REF].split(',')
-        n.set_properties(resource_name=d.get(ABCPropertyGraph.PROP_NAME, None),
-                         resource_type=n.type_from_str(d.get(ABCPropertyGraph.PROP_TYPE, None)),
+        n.set_properties(name=d.get(ABCPropertyGraph.PROP_NAME, None),
+                         type=n.type_from_str(d.get(ABCPropertyGraph.PROP_TYPE, None)),
+                         model=d.get(ABCPropertyGraphConstants.PROP_MODEL, None),
                          capacities=Capacities().from_json(d.get(ABCPropertyGraph.PROP_CAPACITIES, None)),
                          labels=Labels().from_json(d.get(ABCPropertyGraph.PROP_LABELS, None)),
+                         capacity_allocations=Capacities().from_json(d.get(ABCPropertyGraph.PROP_CAPACITY_ALLOCATIONS,
+                                                                           None)),
+                         label_allocations=Labels().from_json(d.get(ABCPropertyGraph.PROP_LABEL_ALLOCATIONS,
+                                                                    None)),
+                         reservation_info=ReservationInfo().from_json(d.get(ABCPropertyGraph.PROP_RESERVATION_INFO,
+                                                                            None)),
                          capacity_delegations=Delegation.from_json_to_list(
                              d.get(ABCPropertyGraph.PROP_CAPACITY_DELEGATIONS, None)),
                          label_delegations=Delegation.from_json_to_list(
@@ -367,7 +624,36 @@ class ABCPropertyGraph(ABCPropertyGraphConstants):
                          management_ip=d.get(ABCPropertyGraph.PROP_MGMT_IP, None),
                          allocation_constraints=d.get(ABCPropertyGraph.PROP_ALLOCATION_CONSTRAINTS, None),
                          service_endpoint=d.get(ABCPropertyGraph.PROP_SERVICE_ENDPOINT, None),
-                         details=d.get(ABCPropertyGraph.PROP_DETAILS, None)
+                         details=d.get(ABCPropertyGraph.PROP_DETAILS, None),
+                         node_map=json.loads(d[ABCPropertyGraph.PROP_NODE_MAP]) if
+                         d.get(ABCPropertyGraph.PROP_NODE_MAP, None) is not None else None
+                         )
+        return n
+
+    @staticmethod
+    def link_sliver_from_graph_properties_dict(d: Dict[str, str]) -> NetworkLinkSliver:
+        n = NetworkLinkSliver()
+        n.set_properties(name=d.get(ABCPropertyGraph.PROP_NAME, None),
+                         type=n.type_from_str(d.get(ABCPropertyGraph.PROP_TYPE, None)),
+                         model=d.get(ABCPropertyGraphConstants.PROP_MODEL, None),
+                         capacities=Capacities().from_json(d.get(ABCPropertyGraph.PROP_CAPACITIES, None)),
+                         labels=Labels().from_json(d.get(ABCPropertyGraph.PROP_LABELS, None)),
+                         capacity_allocations=Capacities().from_json(d.get(ABCPropertyGraph.PROP_CAPACITY_ALLOCATIONS,
+                                                                           None)),
+                         label_allocations=Labels().from_json(d.get(ABCPropertyGraph.PROP_LABEL_ALLOCATIONS,
+                                                                    None)),
+                         reservation_info=ReservationInfo().from_json(d.get(ABCPropertyGraph.PROP_RESERVATION_INFO,
+                                                                            None)),
+                         capacity_delegations=Delegation.from_json_to_list(
+                             d.get(ABCPropertyGraph.PROP_CAPACITY_DELEGATIONS, None)),
+                         label_delegations=Delegation.from_json_to_list(
+                             d.get(ABCPropertyGraph.PROP_LABEL_DELEGATIONS, None)),
+                         allocation_constraints=d.get(ABCPropertyGraph.PROP_ALLOCATION_CONSTRAINTS, None),
+                         details=d.get(ABCPropertyGraph.PROP_DETAILS, None),
+                         layer=d.get(ABCPropertyGraph.PROP_LAYER),
+                         technology=d.get(ABCPropertyGraph.PROP_TECHNOLOGY),
+                         node_map=json.loads(d[ABCPropertyGraph.PROP_NODE_MAP]) if
+                         d.get(ABCPropertyGraph.PROP_NODE_MAP, None) is not None else None
                          )
         return n
 
@@ -380,14 +666,17 @@ class ABCPropertyGraph(ABCPropertyGraphConstants):
         """
         cs = ComponentSliver()
         cs.set_properties(name=d.get(ABCPropertyGraph.PROP_NAME, None),
-                          resource_type=cs.type_from_str(d.get(ABCPropertyGraph.PROP_TYPE, None)),
+                          type=cs.type_from_str(d.get(ABCPropertyGraph.PROP_TYPE, None)),
+                          model=d.get(ABCPropertyGraphConstants.PROP_MODEL, None),
                           capacities=Capacities().from_json(d.get(ABCPropertyGraph.PROP_CAPACITIES, None)),
                           labels=Labels().from_json(d.get(ABCPropertyGraph.PROP_LABELS, None)),
                           capacity_delegations=Delegation.from_json_to_list(
                               d.get(ABCPropertyGraph.PROP_CAPACITY_DELEGATIONS, None)),
                           label_delegations=Delegation.from_json_to_list(
                               d.get(ABCPropertyGraph.PROP_LABEL_DELEGATIONS, None)),
-                          details=d.get(ABCPropertyGraph.PROP_DETAILS, None)
+                          details=d.get(ABCPropertyGraph.PROP_DETAILS, None),
+                          node_map=json.loads(d[ABCPropertyGraph.PROP_NODE_MAP]) if
+                          d.get(ABCPropertyGraph.PROP_NODE_MAP, None) is not None else None
                           )
         return cs
 
@@ -400,7 +689,8 @@ class ABCPropertyGraph(ABCPropertyGraphConstants):
         """
         sf = SwitchFabricSliver()
         sf.set_properties(name=d.get(ABCPropertyGraph.PROP_NAME, None),
-                          resource_type=sf.type_from_str(d.get(ABCPropertyGraph.PROP_TYPE, None)),
+                          type=sf.type_from_str(d.get(ABCPropertyGraph.PROP_TYPE, None)),
+                          model=d.get(ABCPropertyGraphConstants.PROP_MODEL, None),
                           capacities=Capacities().from_json(d.get(ABCPropertyGraph.PROP_CAPACITIES, None)),
                           labels=Labels().from_json(d.get(ABCPropertyGraph.PROP_LABELS, None)),
                           capacity_delegations=Delegation.from_json_to_list(
@@ -408,7 +698,9 @@ class ABCPropertyGraph(ABCPropertyGraphConstants):
                           label_delegations=Delegation.from_json_to_list(
                               d.get(ABCPropertyGraph.PROP_LABEL_DELEGATIONS, None)),
                           layer=sf.layer_from_str(d.get(ABCPropertyGraph.PROP_LAYER, None)),
-                          details=d.get(ABCPropertyGraph.PROP_DETAILS, None)
+                          details=d.get(ABCPropertyGraph.PROP_DETAILS, None),
+                          node_map=json.loads(d[ABCPropertyGraph.PROP_NODE_MAP]) if
+                          d.get(ABCPropertyGraph.PROP_NODE_MAP, None) is not None else None
                           )
         return sf
 
@@ -421,14 +713,17 @@ class ABCPropertyGraph(ABCPropertyGraphConstants):
         """
         isl = InterfaceSliver()
         isl.set_properties(name=d.get(ABCPropertyGraph.PROP_NAME, None),
-                           resource_type=isl.type_from_str(d.get(ABCPropertyGraph.PROP_TYPE, None)),
+                           type=isl.type_from_str(d.get(ABCPropertyGraph.PROP_TYPE, None)),
+                           model=d.get(ABCPropertyGraphConstants.PROP_MODEL, None),
                            capacities=Capacities().from_json(d.get(ABCPropertyGraph.PROP_CAPACITIES, None)),
                            labels=Labels().from_json(d.get(ABCPropertyGraph.PROP_LABELS, None)),
                            capacity_delegations=Delegation.from_json_to_list(
                                d.get(ABCPropertyGraph.PROP_CAPACITY_DELEGATIONS, None)),
                            label_delegations=Delegation.from_json_to_list(
                                d.get(ABCPropertyGraph.PROP_LABEL_DELEGATIONS, None)),
-                           details=d.get(ABCPropertyGraph.PROP_DETAILS, None)
+                           details=d.get(ABCPropertyGraph.PROP_DETAILS, None),
+                           node_map=json.loads(d[ABCPropertyGraph.PROP_NODE_MAP]) if
+                           d.get(ABCPropertyGraph.PROP_NODE_MAP, None) is not None else None
                            )
         return isl
 
@@ -467,6 +762,33 @@ class ABCPropertyGraph(ABCPropertyGraphConstants):
 
         return ns
 
+    def build_deep_link_sliver(self, *, node_id: str) -> NetworkLinkSliver:
+        """
+        Build a deep NetworkLinkSliver from a graph node
+        :param node_id:
+        :return:
+        """
+        clazzes, props = self.get_node_properties(node_id=node_id)
+        if ABCPropertyGraph.CLASS_Link not in clazzes:
+            raise PropertyGraphQueryException(node_id=node_id, graph_id=self.graph_id,
+                                              msg="Node is not of class Link")
+        # create top-level sliver
+        ns = self.link_sliver_from_graph_properties_dict(props)
+        ns.node_id = node_id
+        # find interfaces and attach
+        ifs = self.get_first_neighbor(node_id=node_id, rel=ABCPropertyGraph.REL_CONNECTS,
+                                      node_label=ABCPropertyGraph.CLASS_ConnectionPoint)
+        if ifs is not None and len(ifs) > 0:
+            ifi = InterfaceInfo()
+            for i in ifs:
+                _, iprops = self.get_node_properties(node_id=i)
+                ifsl = self.interface_sliver_from_graph_properties_dict(iprops)
+                ifsl.node_id = node_id
+                ifi.add_interface(ifsl)
+            ns.interface_info = ifi
+
+        return ns
+
     def build_deep_sf_sliver(self, *, node_id: str) -> SwitchFabricSliver:
         clazzes, props = self.get_node_properties(node_id=node_id)
         if ABCPropertyGraph.CLASS_SwitchFabric not in clazzes:
@@ -552,6 +874,14 @@ class ABCGraphImporter(ABC):
         :return:
         """
 
+    @abstractmethod
+    def cast_graph(self, *, graph_id: str) -> ABCPropertyGraph:
+        """
+        Recast a graph with a given node id, checks that graph exists
+        :param graph_id:
+        :return:
+        """
+
     @staticmethod
     def enumerate_graph_nodes(*, graph_file: str, new_graph_file: str, node_id_prop: str = "NodeID") -> None:
         """
