@@ -69,8 +69,11 @@ class Neo4jPropertyGraph(ABCPropertyGraph):
 
         for r in rules_dict:
             with self.driver.session() as session:
-                # print('Applying rule ', r['msg'])
-                v = session.run(r['rule'], graphId=self.graph_id).single().value()
+                #print('Applying rule ', r['msg'])
+                single = session.run(r['rule'], graphId=self.graph_id).single()
+                if single is None:
+                    return
+                v = single.value()
                 # print("Rule {}, value {}".format(r['msg'], v))
                 if v is False:
                     raise PropertyGraphImportException(graph_id=self.graph_id, msg=r['msg'])
@@ -94,6 +97,16 @@ class Neo4jPropertyGraph(ABCPropertyGraph):
         with self.driver.session() as session:
             session.run('match (n:GraphNode {GraphID: $graphId })detach delete n', graphId=self.graph_id)
 
+    def get_all_nodes_by_class(self, *, label: str) -> List[str]:
+        assert label is not None
+        query = f"MATCH (n:{label} {{GraphID: $graphId}}) RETURN collect(n.NodeID) as nodeids"
+        with self.driver.session() as session:
+            val = session.run(query, graphId=self.graph_id).single()
+            if val is None:
+                raise PropertyGraphQueryException(graph_id=self.graph_id,
+                                                  node_id=None, msg="Unable to find network links")
+            return val.data()['nodeids']
+
     def list_all_node_ids(self) -> List[str]:
         """
         List all NodeID properties of nodes in a graph
@@ -181,7 +194,22 @@ class Neo4jPropertyGraph(ABCPropertyGraph):
             val = session.run(query, graphId=self.graph_id, nodeId=node_id, propVal=prop_val)
             if val is None or len(val.value()) == 0:
                 raise PropertyGraphQueryException(graph_id=self.graph_id,
-                                                  node_id=node_id, msg="Unable to set property")
+                                                  node_id=node_id, msg=f"Unable to set property {prop_name}")
+
+    def unset_node_property(self, *, node_id: str, prop_name: str) -> None:
+        assert node_id is not None
+        assert prop_name is not None
+
+        if prop_name in ABCPropertyGraph.NO_UNSET_PROPERTIES:
+            raise PropertyGraphQueryException(graph_id=self.graph_id, node_id=node_id,
+                                              msg=f"Not allowed to unset property {prop_name}")
+        query = f"MATCH (n:GraphNode {{GraphID: $graphId, NodeID: $nodeId}}) " \
+                f"REMOVE n.{prop_name} RETURN n.NodeID"
+        with self.driver.session() as session:
+            val = session.run(query, graphId=self.graph_id, nodeId=node_id)
+            if val is None or len(val.value()) == 0:
+                raise PropertyGraphQueryException(graph_id=self.graph_id, node_id=node_id,
+                                                  msg=f"Unable to unset property {prop_name}")
 
     def update_nodes_property(self, *, prop_name: str, prop_val: Any) -> None:
         """
@@ -198,7 +226,7 @@ class Neo4jPropertyGraph(ABCPropertyGraph):
             val = session.run(query, graphId=self.graph_id, propVal=prop_val)
             if val is None or len(val.value()) == 0:
                 raise PropertyGraphQueryException(graph_id=self.graph_id, node_id=None,
-                                                  msg="Unable to set property on graph nodes")
+                                                  msg=f"Unable to set property {prop_name} on graph nodes")
 
     def update_node_properties(self, *, node_id: str, props: Dict[str, Any]) -> None:
         """
@@ -223,7 +251,7 @@ class Neo4jPropertyGraph(ABCPropertyGraph):
             if val is None or len(val.value()) == 0:
                 raise PropertyGraphQueryException(graph_id=self.graph_id,
                                                   node_id=node_id,
-                                                  msg="Unable to set properties")
+                                                  msg="Unable to set multiple properties")
 
     def update_link_property(self, *, node_a: str, node_b: str, kind: str, prop_name: str,
                              prop_val: Any) -> None:
@@ -242,13 +270,27 @@ class Neo4jPropertyGraph(ABCPropertyGraph):
         assert prop_name is not None
         assert prop_val is not None
         query = f"MATCH (a:GraphNode {{GraphID: $graphId, NodeID: $nodeA}}) -[r:{kind}]- " \
-            f"(b:GraphNode {{GraphID: $graphId, NodeID:$nodeB}}) SET s+={{ {prop_name}: $propVal}} " \
+            f"(b:GraphNode {{GraphID: $graphId, NodeID:$nodeB}}) SET r+={{ {prop_name}: $propVal}} " \
             f"RETURN properties(r)"
         with self.driver.session() as session:
             val = session.run(query, graphId=self.graph_id, nodeA=node_a, nodeB=node_b, propVal=prop_val)
             if val is None or len(val.value()) == 0:
                 raise PropertyGraphQueryException(graph_id=self.graph_id, node_id=node_a,
-                                                  msg="Unable to set property on a link")
+                                                  msg=f"Unable to set property {prop_name} on a link")
+
+    def unset_link_property(self, *, node_a: str, node_b: str, kind: str, prop_name: str) -> None:
+        assert node_a is not None
+        assert node_b is not None
+        assert kind is not None
+        assert prop_name is not None
+
+        query = f"MATCH (a:GraphNode {{GraphID: $graphId, NodeID: $nodeA}}) -[r:{kind}]- " \
+                f"(b:GraphNode {{GraphID: $graphId, NodeID:$nodeB}}) REMOVE r.{prop_name} RETURN r"
+        with self.driver.session() as session:
+            val = session.run(query, graphId=self.graph_id, nodeA=node_a, nodeB=node_b)
+            if val is None or len(val.value()) == 0:
+                raise PropertyGraphQueryException(graph_id=self.graph_id, node_id=node_a,
+                                                  msg=f"Unable to set property {prop_name} on a link")
 
     def update_link_properties(self, *, node_a: str, node_b: str, kind: str,
                                props: Dict[str, Any]) -> None:
@@ -272,7 +314,7 @@ class Neo4jPropertyGraph(ABCPropertyGraph):
             all_props = all_props[:-2]
 
         query = "MATCH (a:GraphNode {{GraphID: $graphId, NodeID: $nodeA}}) -[r:{kind}]- " \
-            f"(b:GraphNode {{GraphID: $graphId, NodeID:$nodeB}}) SET s+= {{ {all_props} }} RETURN properties(s)"
+            f"(b:GraphNode {{GraphID: $graphId, NodeID:$nodeB}}) SET r+= {{ {all_props} }} RETURN properties(s)"
         with self.driver.session() as session:
             val = session.run(query, graphId=self.graph_id, nodeA=node_a, nodeB=node_b)
             if val is None or len(val.value()) == 0:
@@ -451,7 +493,7 @@ class Neo4jPropertyGraph(ABCPropertyGraph):
         assert rel is not None
         assert node_b is not None
 
-        all_props = f", "
+        all_props = ""
         if props is not None:
             for k, v in props.items():
                 all_props += f"{k}: '{v}', "
@@ -778,4 +820,11 @@ class Neo4jGraphImporter(ABCGraphImporter):
         """
         self.log.debug(f'Deleting graph {graph_id}')
         with self.driver.session() as session:
-            session.run('match (n:GraphNode {GraphID: $graphId })detach delete n', graphId=graph_id)
\ No newline at end of file
+            session.run('match (n:GraphNode {GraphID: $graphId })detach delete n', graphId=graph_id)
+
+    def cast_graph(self, *, graph_id: str) -> ABCPropertyGraph:
+
+        assert graph_id is not None
+        neo4jg = Neo4jPropertyGraph(graph_id=graph_id, importer=self, logger=self.log)
+        assert neo4jg.graph_exists()
+        return neo4jg
