@@ -26,11 +26,11 @@
 """
 Base class for all sliver types
 """
-from typing import Any, List
+from typing import Any, Tuple
 from abc import ABC, abstractmethod
 
-from fim.slivers.capacities_labels import Capacities, Labels
-from fim.slivers.delegations import Delegation, Pool
+from fim.slivers.capacities_labels import Capacities, Labels, ReservationInfo
+from fim.slivers.delegations import Delegation
 
 
 class BaseSliver(ABC):
@@ -45,53 +45,28 @@ class BaseSliver(ABC):
         self.labels = None
         self.capacity_delegations = None
         self.label_delegations = None
+        self.capacity_allocations = None
+        self.label_allocations = None
+        self.reservation_info = None
         self.node_id = None
         self.details = None
+        self.node_map = None
 
-    def set_resource_type(self, resource_type):
-        """
-        Could be NodeType, ComponentType, InterfaceType etc
-        :param resource_type:
-        :return:
-        """
+    def set_type(self, resource_type):
         self.resource_type = resource_type
 
-    def get_resource_type(self):
-        return self.resource_type
-
     def get_type(self):
-        """
-        Aliased to get_resource_type
-        :return:
-        """
-        return self.get_resource_type()
-
-    def set_resource_name(self, resource_name: str):
-        self.resource_name = resource_name
+        return self.resource_type
 
     def set_name(self, resource_name: str):
         self.resource_name = resource_name
 
-    def get_resource_name(self):
-        return self.resource_name
-
     def get_name(self):
         return self.resource_name
 
-    def set_resource_model(self, resource_model: str):
-        self.resource_model = resource_model
-
     def set_model(self, resource_model: str):
-        """
-        Aliased to set_resource_model
-        :param resource_model:
-        :return:
-        """
         self.resource_model = resource_model
 
-    def get_resource_model(self):
-        return self.resource_model
-
     def get_model(self):
         return self.resource_model
 
@@ -119,12 +94,36 @@ class BaseSliver(ABC):
     def get_label_delegations(self) -> Delegation:
         return self.label_delegations
 
+    def set_label_allocations(self, lab: Labels) -> None:
+        self.label_allocations = lab
+
+    def get_label_allocations(self) -> Labels:
+        return self.label_allocations
+
+    def set_capacity_allocations(self, cap: Capacities) -> None:
+        self.capacity_allocations = cap
+
+    def get_capacity_allocations(self) -> Capacities:
+        return self.capacity_allocations
+
+    def set_reservation_info(self, ri: ReservationInfo) -> None:
+        self.reservation_info = ri
+
+    def get_reservation_info(self) -> ReservationInfo:
+        return self.reservation_info
+
     def set_details(self, desc: str) -> None:
         self.details = desc
 
     def get_details(self) -> str:
         return self.details
 
+    def set_node_map(self, node_map: Tuple[str, str]) -> None:
+        self.node_map = node_map
+
+    def get_node_map(self) -> Tuple[str, str] or None:
+        return tuple(self.node_map) if self.node_map is not None else None
+
     def set_properties(self, **kwargs):
         """
         Lets you set multiple properties exposed via setter methods
@@ -140,7 +139,7 @@ class BaseSliver(ABC):
                 raise RuntimeError(f'Unable to set property {k} on the sliver - no such property available')
 
     @classmethod
-    def list_properties(cls):
+    def list_properties(cls) -> Tuple[str]:
         """
         List properties available for setting/getting on a sliver (those exposing
         setters)
@@ -151,7 +150,7 @@ class BaseSliver(ABC):
         for k in dir(cls):
             if k.startswith('set_') and k not in exclude_set:
                 ret.append(k[4:])
-        return ret
+        return tuple(ret)
 
     def set_property(self, prop_name: str, prop_val: Any):
         """
