@@ -100,6 +100,8 @@ class Capacities(JSONField):
     Implements basic capacity field handling - encoding and decoding
     from JSON dictionaries of properties
     """
+    UNITS = {'cpu': '', 'unit': '', 'core': '', 'ram': 'G', 'disk': 'G', 'bw': 'G'}
+
     def __init__(self):
         self.cpu = 0
         self.core = 0
@@ -117,7 +119,8 @@ class Capacities(JSONField):
         :return: self to support call chaining
         """
         for k, v in kwargs.items():
-            assert v >= 0
+            if v is not None:
+                assert v >= 0
             try:
                 # will toss an exception if field is not defined
                 self.__getattribute__(k)
@@ -126,6 +129,31 @@ class Capacities(JSONField):
                 raise RuntimeError(f"Unable to set field {k} of capacity, no such field available")
         return self
 
+    def __add__(self, other):
+        assert isinstance(other, Capacities)
+        ret = Capacities()
+
+        ret.cpu = self.cpu + other.cpu
+        ret.core = self.core + other.core
+        ret.ram = self.ram + other.ram
+        ret.disk = self.disk + other.disk
+        ret.bw = self.bw + other.bw
+        ret.unit = self.unit + other.unit
+        return ret
+
+    def __repr__(self):
+        return self.to_json()
+
+    def __str__(self):
+        d = self.__dict__.copy()
+        for k in self.__dict__:
+            if d[k] is None or d[k] == 0:
+                d.pop(k)
+        ret = "{ "
+        for i, v in d.items():
+            ret = ret + i + ": " + str(v) + self.UNITS[i] + " "
+        return ret + "}"
+
 
 class Labels(JSONField):
     """
@@ -142,7 +170,42 @@ class Labels(JSONField):
         self.asn = None
         self.vlan = None
         self.vlan_range = None
-        self.node = None
+        self.instance = None
+        self.instance_parent = None
+
+    def set_fields(self, **kwargs):
+        """
+        Universal setter for all fields. Values should be strings or lists of strings.
+        Throws a RuntimeError if you try to set a non-existent field.
+        :param kwargs:
+        :return: self to support call chaining
+        """
+        for k, v in kwargs.items():
+            assert v is not None  # could be strings or lists of strings
+            assert isinstance(v, str) or isinstance(v, list)
+            try:
+                # will toss an exception if field is not defined
+                self.__getattribute__(k)
+                self.__setattr__(k, v)
+            except AttributeError:
+                raise RuntimeError(f"Unable to set field {k} of labels, no such field available")
+        # to support call chaining
+        return self
+
+    def __add__(self, other):
+        assert isinstance(other, Labels)
+        # FIXME: does ADD mean finding a union of label sets per type?
+        raise RuntimeError("Not Implemented")
+
+
+class ReservationInfo(JSONField):
+    """
+    Reservation info structure for ASM sliver objects
+    """
+
+    def __init__(self):
+        self.reservation_id = None
+        self.reservation_state = None
 
     def set_fields(self, **kwargs):
         """
@@ -160,6 +223,6 @@ class Labels(JSONField):
                 self.__getattribute__(k)
                 self.__setattr__(k, v)
             except AttributeError:
-                raise RuntimeError(f"Unable to set field {k} of labels, no such field available")
+                raise RuntimeError(f"Unable to set field {k} of reservation info, no such field available")
         # to support call chaining
         return self
